/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

#include "qpid/RefCountedBuffer.h"
#include <stdlib.h>
#include <new>

namespace qpid {

void RefCountedBuffer::released() const {
    this->~RefCountedBuffer();
    ::free (reinterpret_cast<void *>(const_cast<RefCountedBuffer *>(this)));
}

BufferRef RefCountedBuffer::create(size_t n) {
    void* store=::malloc (n + sizeof(RefCountedBuffer));
    if (NULL == store)
        throw std::bad_alloc();
    new(store) RefCountedBuffer;
    char* start = reinterpret_cast<char *>(store) + sizeof(RefCountedBuffer);
    return BufferRef(
        boost::intrusive_ptr<RefCounted>(reinterpret_cast<RefCountedBuffer*>(store)),
        start, start+n);
}

} // namespace qpid


